#!/usr/bin/env python3

"""About module for kumaone"""

# Import builtin python libraries
from datetime import datetime

year = datetime.now().year

__version__ = "0.0.1a1"
__author__ = "Dalwar Hossain"
__license__ = "BSD 3-Clause"
__home_page__ = "https://kumaone.rtfd.io"
__copyright__ = f"{year}, {__author__}"



#!/usr/bin/env python3

"""Event handler module for kumaone"""

# Import builtin python libraries
from contextlib import contextmanager
from copy import deepcopy
import time

# Import external python libraries
from rich.console import Console
from socketio.exceptions import TimeoutError

# Import custom (local) python packages
from . import ioevents
from .settings import event_data, monitor_events

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()


@contextmanager
def wait_for_event(event):
    """
    Waits for event

    :param event: (str) SocketIO event name.
    :return: None
    """

    try:
        yield
    except:
        raise
    else:
        timestamp = time.time()
        while event_data[event] is None:
            if time.time() - timestamp > 30:
                raise TimeoutError("Event response timed out.")
            time.sleep(0.01)


def get_event_data(event):
    """
    Get event data

    :param event: (event) socketIO event
    :return: (dict) socketIO event data
    """

    timestamp = time.time()
    while event_data[event] is None:
        if time.time() - timestamp > 30:
            raise TimeoutError("Event response timed out.")
        if event_data[event] == {} and event in monitor_events:
            return []
        time.sleep(0.01)
    time.sleep(0.2)
    return deepcopy(event_data[event].copy())


def connect_event():
    """
    Connect event handler

    :return:
    """

    pass


def disconnect_event():
    """
    Disconnect event handler

    :return: None
    """

    pass


def monitor_list_event(data):
    """
    Get all the monitors

    :param data: (dict) Event data.
    :return: None
    """

    event_data[ioevents.monitor_list] = data


def status_page_list_event(data):
    """
    Get all the status pages

    :param data: (dict) Event data.
    :return: None
    """

    event_data[ioevents.status_page_list] = data


def notification_list_event(data):
    """
    Get all notification processes

    :param data: (dict) Event data
    :return: None
    """

    event_data[ioevents.notification_list] = data



#!/usr/bin/env python3

"""Notification settings module for kumaone"""

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

notification_providers = {
    "rocket.chat": {
        "rocketchannel": "",
        "rocketusername": "",
        "rocketiconemo": "",
        "rocketwebhookURL": "",
    },
    "slack": {
        "slackchannelnotify": "",
        "slackchannel": "",
        "slackusername": "",
        "slackiconemo": "",
        "slackwebhookURL": "",
    },
    "teams": {
        "webhookUrl": "",
    },
    "webhook": {
        "webhookContentType": "",
        "webhookCustomBody": "",
        "webhookAdditionalHeaders": "",
        "webhookURL": "",
    },
}



#!/usr/bin/env python3

"""Settings module for kumaone"""

# Import custom (local) python packages
from . import ioevents as event

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

accepted_status_codes = ["200-299"]

authentication_methods = [
    "basic",
    "mtls",
    "none",
    "ntlm",
    "oauth2-cc",
]

docker_engine_connection_types = ["socket", "tcp"]

event_data = {
    event.api_key_list: None,
    event.auto_login: None,
    event.avg_ping: None,
    event.cert_info: None,
    event.connect: None,
    event.disconnect: None,
    event.docker_host_list: None,
    event.heartbeat: None,
    event.heartbeat_list: None,
    event.important_heartbeat_list: None,
    event.info: None,
    event.init_server_timezone: None,
    event.maintenance_list: None,
    event.monitor_list: None,
    event.notification_list: None,
    event.proxy_list: None,
    event.status_page_list: None,
    event.uptime: None,
}

incident_styles = [
    "danger",
    "dark",
    "info",
    "light",
    "primary",
    "warning",
]

maintenance_strategies = [
    "cron",
    "manual",
    "recurring-interval",
    "recurring-weekday",
    "recurring-day-of-month",
    "single",
]

monitor_events = ["avgPing", "uptime", "heartbeatList", "importantHeartbeatList", "certInfo", "heartbeat"]

monitor_status_mapping = {"down": 0, "maintenance": 3, "pending": 2, "up": 1}

monitor_types = [
    "dns",
    "docker",
    "gamedig",
    "group",
    "grpc-keyword",
    "http",
    "json-query",
    "kafka-producer",
    "keyword",
    "mongodb",
    "mqtt",
    "mysql",
    "ping",
    "port",
    "postgres",
    "push",
    "radius",
    "real-browser",
    "redis",
    "sqlserver",
    "stream",
    "tailscale-ping",
]

proxy_protocols = ["http", "https", "socks", "socks4", "socks5", "socks5h"]

timeout = 10


def required_arguments_by_type(monitor_type=None):
    """
    Returns minimum required arguments by event type
    :param monitor_type: (str) Name of the event type
    :return:
    """

    required_arguments = {
        "dns": ["hostname", "dns_resolve_server", "port"],
        "docker": ["docker_container", "docker_host"],
        "gamedig": ["game", "hostname", "port"],
        "group": [],
        "grpc-keyword": ["grpcUrl", "keyword", "grpcServiceName", "grpcMethod"],
        "http": ["url", "maxredirects"],
        "json-query": ["url", "jsonPath", "expectedValue"],
        "kafka-producer": ["kafkaProducerTopic", "kafkaProducerMessage"],
        "keyword": ["url", "keyword", "maxredirects"],
        "mongodb": [],
        "mqtt": ["hostname", "port", "mqttTopic"],
        "mysql": [],
        "ping": ["hostname"],
        "port": ["hostname", "port"],
        "postgres": [],
        "push": [],
        "radius": [],
        "real-browser": ["hostname", "port"],
        "redis": [],
        "sqlserver": [],
        "stream": ["hostname", "port"],
        "tailscale-ping": ["hostname"],
    }

    return required_arguments[monitor_type] + ["name", "type"]


def get_missing_arguments(input_data=None):
    """
    Finds missing argument key from input based on required argument keys for monitors

    :param input_data: (dict) Monitor payload keys with values
    :return: (list) Missing keys as list or empty list if all required arguments are provided
    """

    required_argument_keys = required_arguments_by_type(monitor_type=input_data["type"])
    missing_arguments = []
    for required_argument in required_argument_keys:
        if input_data.get(required_argument) is None:
            missing_arguments.append(required_argument)
    return missing_arguments



#!/usr/bin/env python3

"""Connection module for kumaone"""

# Import builtin python libraries
import sys

# Import builtin python libraries
from types import SimpleNamespace

# Import external python libraries
from rich.console import Console

import socketio
from socketio.exceptions import TimeoutError

# Import custom (local) python packages
from .event_handlers import (
    connect_event,
    disconnect_event,
    monitor_list_event,
    notification_list_event,
    status_page_list_event,
)
from . import ioevents

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()
sio = socketio.Client(logger=False, engineio_logger=False)
monitor_list_data = None


def _register_event_handlers():
    """
    Registers event handlers

    :return: None
    """

    # sio.on(ioevents.api_key_list, api_key_list_event)
    # sio.on(ioevents.auto_login, auto_login_event)
    # sio.on(ioevents.avg_ping, avg_ping_event)
    # sio.on(ioevents.cert_info, cert_info_event)
    sio.on(ioevents.connect, connect_event)
    sio.on(ioevents.disconnect, disconnect_event)
    # sio.on(ioevents.docker_host_list, docker_host_list_event)
    # sio.on(ioevents.heartbeat, heartbeat_event)
    # sio.on(ioevents.heartbeat_list, heartbeat_list_event)
    # sio.on(ioevents.important_heartbeat_list, important_heartbeat_list_event)
    # sio.on(ioevents.info, info_events)
    # sio.on(ioevents.init_server_timezone, init_server_timezone_event)
    # sio.on(ioevents.maintenance_list, maintenance_list_event)
    sio.on(ioevents.monitor_list, monitor_list_event)
    sio.on(ioevents.notification_list, notification_list_event)
    # sio.on(ioevents.proxy_list, proxy_list_event)
    sio.on(ioevents.status_page_list, status_page_list_event)
    # sio.on(ioevents.uptime, uptime_event)


def connect_login(config_data=None, headers=None):
    """
    Connects to uptime kuma server

    :param config_data: (dict) Uptime kuma server configs
    :param headers: (dict) Custom headers
    :return: (event) Connection event to uptime kuma server
    """

    try:
        # console.print(Rule(title="Connect", style="purple"))
        _register_event_handlers()
        sio.connect(config_data.url, headers=headers)
        console.print(f":pretzel: Connected to {config_data.url}", style="logging.level.info")
    except TimeoutError:
        console.print(f":hourglass_done: Connection timed out.", style="logging.level.info")
    except Exception as err:
        connect_error(err)
    try:
        login_data = {"username": config_data.user, "password": config_data.password}
        login_response = sio.call("login", data=login_data)
        if isinstance(login_response, dict) and "ok" in login_response:
            console.print(f":locked_with_key: Successfully logged in.", style="green")
            login_response = SimpleNamespace(**login_response)
    except Exception as err:
        console.print(f":x:  Error: {err}", style="logging.level.error")
        sys.exit(1)


@sio.event
def connect_error(data):
    console.print(f":x: Could not connect to server. Error: {data}", style="logging.level.error")
    sys.exit(1)


def disconnect():
    # console.print(Rule(title="Disconnect", style="purple"))
    try:
        sio.disconnect()
        console.print(f":firecracker: Disconnected from server.", style="logging.level.info")
    except Exception as err:
        console.print(f":x:  Could not disconnect from server. Error: {err}", style="logging.level.error")
        sys.exit(1)



#!/usr/bin/env python3

"""Utility module for kumaone"""

# Import builtin python libraries
import logging
import os
from pathlib import Path
import sys
import yaml

# Import external python libraries
from rich import print
from rich.console import Console
from rich.logging import RichHandler
import typer

# Import custom (local) python packages
from .connection import sio
from src.kumaone.__about__ import __author__ as author
from src.kumaone.__about__ import __copyright__ as app_copy_right
from src.kumaone.__about__ import __home_page__ as homepage
from src.kumaone.__about__ import __license__ as app_license
from src.kumaone.__about__ import __version__ as version

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()


def version_callback(value: bool):
    if value:
        print(f"{version}")
        raise typer.Exit()


# Turn on/off logging visibility
def log_manager(log_level=None):
    """
    This function acts accordingly with --debug switch

    :param log_level: (str) Log level
    :return: (object) logger
    """

    if log_level is None or log_level == "NOTSET":
        logger = logging.getLogger(__name__)
        logger.addHandler(logging.NullHandler())
        logger.propagate = False
        return logger
    else:
        log_format = "[%(funcName)s()] %(message)s"
        logging.basicConfig(
            level="NOTSET", format=log_format, datefmt="[%m-%d-%Y %X]", handlers=[RichHandler(rich_tracebacks=True)]
        )
        level = logging.getLevelName(f"{log_level}".upper())
        if not log_level.upper().startswith("W"):
            console.print(f":beetle: {log_level} mode is ON.".upper(), style="logging.keyword")
        logger = logging.getLogger(__name__)
        logger.setLevel(level)
        logger.propagate = True
        return logger


def app_info(log_level=None):
    """
    Shows application information

    :param log_level: (str) Log levels
    :return: application information
    """

    logger = log_manager(log_level=log_level)

    logger.info(f"Please check github repository for updated info.")
    # console.print(Rule(style="purple"))
    print(f":hatching_chick: Author: {author}")
    print(f":penguin: Version: {version}")
    print(f":memo: License: {app_license}")
    print(f":link: Home: {homepage}")
    print(f":copyright: Copyright: {app_copy_right}")
    # console.print(Rule(style="purple"))


def _sio_call(event=None, data=None):
    """
    Calls socketIO event

    :param event: (str) Event name.
    :param data: (any) Event related data.
    :return: (any)
    """

    try:
        response = sio.call(event, data=data)
    except TimeoutError:
        console.print(f":hourglass:  Request timed out while waiting for '{event}' event.", style="logging.level.info")
        sys.exit(1)
    # if isinstance(response, dict):
    #     if not response["ok"]:
    #         console.print(f":orange_circle: {response.get('msg')}", style="logging.level.warning")
    #         # sys.exit(1)
    return response
    # try:
    #     json_response = json.load(response)
    #     if not json_response["ok"]:
    #         console.print(f":red_circle: Error: {json_response.get('msg')}")
    #     return json_response
    # except ValueError as err:
    #     console.print(f":dizzy_face: Response is not JSON serializable.", style="logging.level.error")
    #     console.print(f":red_circle: Error: {err}")


def _check_data_path(data_path=None, logger=None, key_to_check_for=None):
    """
    Checks data path for monitor input file or directory

    :param data_path: (Path) uptime kuma input data.
    :param logger: (object) logger object.
    :param key_to_check_for: (str) What type of data should be checked.
    :return: (int) Monitor ID.
    """

    # console.print(Rule(style="purple"))
    console.print(f":clipboard: Checking input data path.", style="logging.level.info")
    if Path(data_path).exists():
        if Path(data_path).is_dir():
            logger.info(f"{data_path} is a directory. All yaml files in this directory will be considered.")
            console.print(
                f":file_folder: Directory input detected. Input file directory: '{data_path}'.",
                style="logging.level.info",
            )
            with os.scandir(Path(data_path)) as items:
                data_files = []
                skipped_files = []
                for item in items:
                    if item.is_file():
                        file_type = item.name.split(".")[-1]
                        if file_type in ["yaml", "yml"]:
                            logger.info(f"{item.name} - {item.stat().st_size} bytes.")
                            with open(item) as tmp_read_file:
                                raw_data = yaml.safe_load(tmp_read_file)
                                logger.debug(raw_data)
                                if key_to_check_for in raw_data:
                                    data_files.append(Path(data_path).joinpath(item.name))
                                else:
                                    logger.info(f"{item.name} did not have {key_to_check_for}, skipped.")
                                    pass
                        else:
                            console.print(
                                f":bulb: '.{file_type}' file type is not supported. Skipping '{item.name}'. ",
                                style="logging.level.info",
                            )
                            skipped_files.append(Path(data_path).joinpath(item.name))
                    else:
                        console.print(
                            f":card_index_dividers: Nested directories are not supported. Skipping '{item.name}'.",
                            style="logging.level.info",
                        )
            console.print(
                f":high_brightness: {len(data_files)} files found in supported format.",
                style="logging.level.info",
            )
            logger.debug(f"{data_files}")
            logger.debug(f"{skipped_files}")
            return sorted(data_files)
        elif Path(data_path).is_file():
            logger.info(f"'{data_path}' is a file.")
            console.print(
                f":high_brightness: Single file input detected. Input file: '{data_path}'.", style="logging.level.info"
            )
            with open(data_path) as tmp_read_file:
                raw_data = yaml.safe_load(tmp_read_file)
                logger.debug(raw_data)
                logger.debug(key_to_check_for)
                if key_to_check_for in raw_data:
                    return sorted([data_path])
                else:
                    console.print(
                        f":orange_circle: Provided data file might not contain necessary data. Missing {key_to_check_for} key.",
                        style="logging.level.warning",
                    )
                    sys.exit(1)
    else:
        console.print(f":x:  Data path: '{data_path}', does not exists!", style="logging.level.error")
        sys.exit(1)


def _mutual_exclusivity_check(size=None):
    """
    Checks mutual exclusivity
    :param size: (int) size of the group
    :return: None
    """

    group = set()

    def callback(ctx: typer.Context, param: typer.CallbackParam, value: str):
        # Add cli option to group if it was called with a value
        if value is not None and param.name not in group:
            group.add(param.name)
        if len(group) > size - 1:
            raise typer.BadParameter(f"{param.name} is mutually exclusive with {group.pop()}")
        return value

    return callback



#!/usr/bin/env python3


"""monitor module for kumaone"""
import sys

# Import builtin python libraries
from pathlib import Path
from typing import Optional

# Import external python libraries
from rich.console import Console
import typer
from typing_extensions import Annotated

# Import custom (local) python packages
from src.kumaone.configs import check_config
from src.kumaone.connection import connect_login, disconnect
from src.kumaone.status_pages import add_status_page, delete_status_page, get_satus_page, list_status_pages
from src.kumaone.utils import _check_data_path, log_manager, _mutual_exclusivity_check

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

# Create typer app and turn off debug mode by default
app = typer.Typer()
state = {"log_level": "NOTSET"}
console = Console()


@app.command(name="add", help="Add one or more uptime kuma status page(s).")
def status_page_add(
    status_pages: Annotated[
        Optional[Path],
        typer.Option(
            ...,
            "--pages",
            "-p",
            help="Status page(s) data. Exclusive to '--title' and '--slug'",
            callback=_mutual_exclusivity_check(size=3),
        ),
    ] = None,
    title: Annotated[
        str,
        typer.Option(
            ...,
            "--title",
            "-t",
            help="Title of the status page. '--slug' is required.",
            callback=_mutual_exclusivity_check(size=3),
        ),
    ] = None,
    slug: Annotated[
        str,
        typer.Option(
            ...,
            "--slug",
            "-s",
            help="Slug of the status page. '--title' is required.",
            callback=_mutual_exclusivity_check(size=3),
        ),
    ] = None,
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
    save: Annotated[
        bool, typer.Option(help="Add monitors to a status page. Valid when '--pages' / '-p' set to a directory.")
    ] = False,
):
    """
    Add one or more uptime kuma status page(s).

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)
    if status_pages is None and title is None and slug is None:
        raise typer.BadParameter(
            "At least '--title' / '-t' and '--slug' / '-s' parameters OR '--pages' / '-p' parameter is required."
        )
    elif status_pages:
        status_page_config = "from_file"
    elif title and slug:
        status_page_config = "inline"
    elif title is None or slug is None:
        raise typer.BadParameter("Both '--title' / '-t' and '--slug' / '-s' parametes are required.")
        exit(1)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    if status_page_config == "inline":
        add_status_page(status_page_title=title, status_page_slug=slug, logger=logger)
    elif status_page_config == "from_file":
        status_page_file_paths = _check_data_path(
            data_path=status_pages, logger=logger, key_to_check_for="status_pages"
        )
        add_status_page(status_page_data_files=status_page_file_paths, logger=logger, url=config_data.url, save=save)
    disconnect()


@app.command(name="delete", help="Delete one or more uptime kuma status page(s).")
def status_page_delete(
    status_pages: Annotated[
        Optional[Path],
        typer.Option(..., "--pages", "-p", help="Status page(s) data. Exclusive to '--slug'"),
    ] = None,
    slug: Annotated[str, typer.Option(..., "--slug", "-s", help="Slug of the status page.")] = None,
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Delete one or more uptime kuma status page(s).

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)
    if status_pages is None and slug is None:
        raise typer.BadParameter(
            message="At least one of the parameters '--slug' / '-s' OR '--pages' / '-p' is required."
        )
        sys.exit(1)
    elif status_pages:
        status_page_config = "from_file"
    elif status_pages is None and slug is not None:
        status_page_config = "inline"

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    if status_page_config == "inline":
        delete_status_page(status_page_slug=slug, logger=logger)
    elif status_page_config == "from_file":
        status_page_file_paths = _check_data_path(
            data_path=status_pages, logger=logger, key_to_check_for="status_pages"
        )
        delete_status_page(status_page_data_files=status_page_file_paths, logger=logger)
    disconnect()


@app.command(name="list", help="List all uptime kuma status pages.")
def status_page_list(
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    verbose: Annotated[bool, typer.Option(help="Show verbose output.")] = False,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Lists all uptime kuma status pages.

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    list_status_pages(verbose=verbose, logger=logger)
    disconnect()


@app.command(name="show", help="Show a status page details.")
def status_page_show(
    slug: Annotated[str, typer.Option(..., "--slug", "-s", help="Slug of the status page.")],
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Show a status page details.

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    get_satus_page(slug=slug, logger=logger, url=config_data.url, show_details=True)
    disconnect()


@app.callback()
def status_page_mission_control(log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET"):
    """
    Kumaone status page manager. An uptime kuma status page manager wrapper.
    """

    if log_level:
        state["log_level"] = log_level


if __name__ == "__main__":
    app()



#!/usr/bin/env python3

"""Config module for kumaone"""

# Import builtin python libraries
from pathlib import Path
import sys
from types import SimpleNamespace
import yaml

# Import external python libraries
from rich.console import Console
from rich.prompt import Prompt
from rich.rule import Rule
import validators

# Import custom (local) python packages
from src.kumaone.utils import log_manager

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()

kuma_config_default_locations = [
    Path.home().joinpath(".config/kumaone/kuma.yaml"),
    Path.home().joinpath("kuma.yaml"),
    "./kuma.yaml",
    "/etc/kumaone/kuma.yaml",
]


def __write_config_file(data_to_write=None, file_path=None):
    """
    Writes data to configuration file.

    :param data_to_write: (dict) Data to write to configuration file.
    :param file_path: (Path) File path to write configuration data.
    :return: (file) Creates a file in file system.
    """

    try:
        with open(file_path, "w") as kuma_config:
            yaml.safe_dump(data_to_write, kuma_config)
            return True
    except Exception as err:
        console.print(f":x: {err} Exception occurred.", style="logging.level.error")
        exit(1)


def check_config(config_path=None, log_level=None, logger=None, get_url=False):
    """
    Checks for uptime kuma config

    :param config_path: (Path) configuration yaml file path
    :param log_level: (str) Log level
    :param logger: (object) Logger object for logging
    :param get_url: (bool) Get uptime kuma url
    :return: (object) Python SimpleNamespace object with the config
    """

    if logger is None:
        logger = log_manager(log_level=log_level)
    else:
        logger = logger

    if config_path is None or not config_path:
        for item in kuma_config_default_locations:
            config_file = Path(item)
            logger.info(f"Looking for uptime kuma config file at: {config_file}")
            if Path.exists(config_file):
                if Path.is_file(config_file):
                    return check_config(config_path=config_file, logger=logger, get_url=get_url)
        logger.critical(f"Config file not found.")
        console.print(f":x: Sorry! We couldn't find the config file.", style="logging.level.critical")
        console.print(
            f":speak_no_evil: Please run 'kumaone config --action create' to create a new config file.",
            style="logging.level.info",
        )
    else:
        config_file = Path(config_path)
        if Path.exists(config_file):
            if Path.is_file(config_file):
                with open(config_file, "r") as kuma_config:
                    config_data = yaml.safe_load(kuma_config)
                    logger.info(f"Config file {config_file} found!")
                    logger.debug(f"{config_data}")
                console.print(f":partying_face: Uptime kuma config file found at: {config_file}", style="green")
                return SimpleNamespace(**config_data["kuma"])
            else:
                logger.error(f"Config path is not a file.")
                console.print(f":x: {config_file} is not a file.", style="red")
                exit(1)
        else:
            logger.error(f"Provided config path doesn't exists.")
            console.print(f":x: {config_file} doesn't exists.", style="red")
            exit(1)


def create_config(config_path=None, log_level=None):
    """
    Create a new config for kumaone with necessary info about uptime kuma

    :param config_path: (Path) Custom configuration path.
    :param log_level: (str) Log level
    :return: (file) Cerates a file with configuration
    """

    logger = log_manager(log_level=log_level)

    logger.info(f"Provided location: {config_path}")
    config_data = {}
    if config_path is None or not config_path:
        logger.info(f"Provided config path is either empty or set to None.")
        console.print(
            f":bear: No custom config path provided. Default path will be used for configuration.",
            style="logging.level.info",
        )
        config_file_path = kuma_config_default_locations[0]
    else:
        logger.info(f"Custom config path provided.")
        console.print(
            f":teddy_bear: Custom config path provided. {config_path} will be used for configuration.",
            style="logging.level.info",
        )
        config_file_path = config_path
    logger.info(f"Uptime kuma configuration will be written in: {config_file_path}")
    console.print(f":honey_pot: Creating config file at: {config_file_path}", style="logging.level.info")
    while True:
        config_data["url"] = Prompt.ask(":link: Uptime kuma URL (https://example.com)")
        if validators.url(config_data["url"]):
            logger.info("URL syntax is valid.")
            break
        else:
            console.print(":lying_face: Invalid URL syntax.", style="prompt.invalid")
    config_data["user"] = Prompt.ask(":goat: Uptime kuma username")
    config_data["password"] = Prompt.ask(":see_no_evil: Uptime kuma password", password=True)

    parent_directory = Path(config_file_path).parent
    config_data_to_write = {"kuma": config_data}

    if parent_directory.exists():
        logger.info(f"Parent directory {parent_directory} exists. Creating config file.")
        console.print(":file_folder: Parent directory exists.", style="logging.level.info")
        console.print(f":pencil: Creating configuration file.", style="logging.level.info")
        if __write_config_file(data_to_write=config_data_to_write, file_path=config_file_path):
            logger.info("Configuration file creation was successful.")
            console.print(f":white_heavy_check_mark:  Successfully created the configuration file!", style="green")
    else:
        logger.info(f"Parent directory {parent_directory} doesn't exist. The directory will be created.")
        console.print(
            f":file_folder: Parent directory doesn't exists. {parent_directory} directory will be created.",
            style="logging.level.info",
        )
        try:
            Path(parent_directory).mkdir(parents=True, exist_ok=True)
            console.print(f":open_file_folder: Successfully created {parent_directory} directory.", style="green")
            console.print(f":pencil: Creating configuration file.", style="logging.level.info")
            if __write_config_file(data_to_write=config_data_to_write, file_path=config_file_path):
                logger.info("Configuration file creation was successful.")
                console.print(f":white_heavy_check_mark:  Successfully created the configuration file!", style="green")
        except Exception as err:
            logger.error(f"{err} exception occurred.")
            console.print(f":x: {err} exception occurred.", style="logging.level.error")
            exit(1)


def delete_config(config_path=None, remove_parent=False, log_level=None):
    """
    Delete uptime kuma config

    :param config_path: (Path) Custom configuration path.
    :param remove_parent: (boolean) Remove parent directory if True. Default is False.
    :param log_level: (str) Log level
    :return: None
    """

    logger = log_manager(log_level=log_level)

    logger.info(f"Deleting {config_path} configuration file.")
    files_to_delete = []
    if config_path is None or not config_path:
        logger.info("Empty config path provided.")
        console.print(
            f":zipper-mouth_face: Empty config path provided. Looking for default config file.",
            style="logging.level.info",
        )
        for item in kuma_config_default_locations:
            if Path(item).exists():
                logger.info(f"Config file found at: {item}")
                files_to_delete.append(item)
    else:
        if Path(config_path).exists():
            logger.info(f"Config file found at: {config_path}")
            files_to_delete.append(config_path)
        else:
            logger.info(f"Provided config file doesn't exists!")
            console.print(
                f":lollipop: Provided config file path doesn't exists. Nothing to delete.", style="logging.level.info"
            )
            sys.exit(0)
    if not files_to_delete:
        logger.info(f"No config file found.")
        console.print(f":lollipop: No config file found. Nothing to delete.", style="logging.level.info")
        sys.exit(0)
    else:
        for item in files_to_delete:
            console.print(f":llama: Deleting {item} file.", style="logging.level.info")
            try:
                Path.unlink(item, missing_ok=True)
                logger.info(f"File {item} removed.")
                console.print(f":watermelon: File {item} has been deleted.", style="green")
            except Exception as err:
                console.print(f":x: Exception occurred. ERROR: {err}", style="logging.level.error")
                logger.error(f"{err}")
                sys.exit(1)



#!/usr/bin/env python3

"""Monitors module for kumaone"""

# Import builtin python libraries
import json
import sys

# Import external python libraries
from rich.columns import Columns
from rich.console import Console
from rich import print
from rich.table import Table
import yaml

# Import custom (local) python packages
from .event_handlers import get_event_data, wait_for_event
from . import ioevents
from .payload_handler import _get_monitor_payload
from .settings import get_missing_arguments
from .utils import _sio_call

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()


def _check_monitor(monitor_name_to_check=None):
    """
    Checks if the provided monitor exists or not. Can check both group and process monitor.

    :param monitor_name_to_check: (str) Name of the monitor to check
    :return: (dict) Monitor info dictionary
    """

    current_monitors = get_event_data(ioevents.monitor_list).values()
    monitor_exists = False
    monitor_info = {}
    for item in current_monitors:
        if item["name"] == monitor_name_to_check:
            m_id = item["id"]
            monitor_exists = True
            monitor_info = {"name": item["name"], "id": m_id, "exists": monitor_exists}
        else:
            pass
    if not monitor_exists:
        monitor_info = {"name": monitor_name_to_check, "id": None, "exists": monitor_exists}
    return monitor_info


def _get_or_create_monitor_group(group_name=None):
    """
    Get or create monitor group

    :param group_name: (str) Monitor group to create
    :return: (dict) Name and id of the group as python dict
    """

    monitor_group_check = _check_monitor(monitor_name_to_check=group_name)
    if monitor_group_check['exists']:
        console.print(f":sunflower: Monitor group '{group_name}' already exists.", style="logging.level.info")
        monitor_group_info = {"name": group_name, "id": monitor_group_check["id"]}
    else:
        # logger.info(f":blue_circle: Monitor group: '{group_name}' does not exist.", style="logging.level.info")
        monitor_group_data_payload = _get_monitor_payload(type="group", name=group_name)
        with wait_for_event(ioevents.monitor_list):
            add_event_response = _sio_call("add", monitor_group_data_payload)
        if add_event_response["ok"]:
            monitor_group_info = {"name": group_name, "id": add_event_response["monitorID"]}
            console.print(
                f":hatching_chick: Monitor group '{group_name}' has been created.", style="logging.level.info"
            )
        else:
            console.print(f":red_circle: Error! {add_event_response.get('msg')}", style="logging.level.error")
    return monitor_group_info


def _get_or_create_process_monitor(input_data=None):
    """
    Get or create monitor process

    :param input_data: (dict) Monitor process input data
    :return: (dict) 'add' event response or process exists dictionary
    """

    process_monitor_name = input_data["name"]
    process_monitor_check = _check_monitor(monitor_name_to_check=process_monitor_name)

    if process_monitor_check["exists"]:
        console.print(
            f":sunflower: Monitor process '{process_monitor_name}' already exists.", style="logging.level.info"
        )
        process_monitor_info = {"name": process_monitor_name, "id": process_monitor_check["id"]}
    else:
        # logger.info(f"Monitor process '{process_monitor_name}' doesn't exists.")
        # logger.info(f"Creating monitor process for '{process_monitor_name}'.")
        process_monitor_data_payload = _get_monitor_payload(**input_data)
        # print(process_monitor_data_payload)
        missing_arguments = get_missing_arguments(process_monitor_data_payload)
        if not missing_arguments:
            with wait_for_event(ioevents.monitor_list):
                add_event_response = _sio_call("add", process_monitor_data_payload)
            if add_event_response["ok"]:
                process_monitor_info = {"name": process_monitor_name, "id": add_event_response["monitorID"]}
                console.print(
                    f":hatching_chick: Monitor process for '{input_data['name']}' has been created.",
                    style="logging.level.info",
                )
            else:
                console.print(f":red_circle: Error! {add_event_response.get('msg')}", style="logging.level.error")
        else:
            flat_missing_arguments = ", ".join([f"'{item}'" for item in missing_arguments])
            console.print(
                f":nut_and_bolt: Missing arguments for monitor process '{process_monitor_name}'. Missing {flat_missing_arguments} key(s).",
                style="logging.level.error",
            )
            sys.exit(1)
    return process_monitor_info


def _delete_process_monitor_or_group(input_data=None):
    """
    Delete a monitor

    :param input_data: (dict) Input data about the monitor to be deleted
    :return: (bool) True if deletion is successful, False otherwise
    """

    process_monitor_name = input_data["name"]
    process_monitor_check = _check_monitor(monitor_name_to_check=process_monitor_name)

    if process_monitor_check["exists"]:
        # console.print(f":wastebasket: Deleting process monitor '{process_monitor_name}'.", style="logging.level.info")
        monitor_id = process_monitor_check["id"]
        delete_event_response = _sio_call("deleteMonitor", monitor_id)
        if isinstance(delete_event_response, dict):
            if delete_event_response["ok"]:
                console.print(
                    f":ghost: '{process_monitor_name}' monitor deletion successful!", style="logging.level.info"
                )
            else:
                console.print(
                    f":crab: '{process_monitor_name}' monitor deletion unsuccessful!", style="logging.level.warning"
                )
        else:
            console.print(
                f":red_circle: Something went wrong! {delete_event_response['msg']}", style="logging.level.error"
            )
            return False
    else:
        console.print(
            f":running_shoe: Monitor '{process_monitor_name}' doesn't exist. Skipping...", style="logging.level.info"
        )


def add_monitor(monitor_data_files=None, logger=None):
    """
    Adds one or more monitor(s)

    :param monitor_data_files: (list) Data file path(s)
    :param logger: (object) Logger object
    :return: None
    """

    for monitor_file in monitor_data_files:
        with open(monitor_file) as monitors_:
            monitors = yaml.safe_load(monitors_)["monitors"]
            groups = [group for group in monitors.keys()]
            for group in groups:
                print(f"-" * 38 + f" {group} ".upper() + f"-" * (40 - len(group)))
                if group == "default":
                    monitor_group_info = {"name": None, "id": None}
                else:
                    monitor_group_info = _get_or_create_monitor_group(group_name=group)
                if monitor_group_info:
                    for input_data in monitors[group]:
                        if isinstance(input_data, dict):
                            if group == "default":
                                pass
                            else:
                                input_data.update({"parent": monitor_group_info["id"]})
                            process_monitor_info = _get_or_create_process_monitor(input_data=input_data)
                            if process_monitor_info:
                                pass
                        else:
                            console.print(
                                f":gloves: Monitor process data malformed, please check input.",
                                style="logging.level.error",
                            )
                            sys.exit(1)
                else:
                    console.print(
                        f":potato: Group creation failed! Couldn't create group: '{group}'", style="logging.level.info"
                    )
                    console.print(f":red_circle: Message: {monitor_group_info}", style="logging.level.error")
    print("-" * 80)


def delete_monitor(monitor_data_files=None, monitor_name=None, logger=None):
    """
    Deletes one or more monitor(s)

    :param monitor_data_files: (list) Data file path(s)
    :param monitor_name: (str) Single monitor name
    :param logger: (object) Logger object
    :return: None
    """

    if monitor_name is not None:
        _delete_process_monitor_or_group(input_data={"name": monitor_name})
    elif monitor_data_files:
        for monitor_file in monitor_data_files:
            with open(monitor_file, "r") as monitors_:
                monitors = yaml.safe_load(monitors_)["monitors"]
                groups = [group for group in monitors.keys()]
                for group in groups:
                    print(f"-" * 38 + f" {group} ".upper() + f"-" * (40 - len(group)))
                    for process_monitor_data in monitors[group]:
                        if isinstance(process_monitor_data, dict):
                            _delete_process_monitor_or_group(input_data=process_monitor_data)
                        else:
                            console.print(
                                f":gloves: Monitor process data malformed, please check input.",
                                style="logging.level.error",
                            )
                    _delete_process_monitor_or_group(input_data={"name": group})
        print("-" * 80)


def list_monitors(monitor_id=None, show_groups=None, show_processes=None, verbose=None, logger=None):
    """
    Show list of monitor groups and processes.

    :param monitor_id: (int) Monitor ID.
    :param show_groups: (bool) Show only monitoring groups.
    :param show_processes: (bool) Show only monitoring processes.
    :param verbose: (bool) Show verbose output.
    :param logger: (object) Logger object.
    :return: None
    """

    response = list(get_event_data(ioevents.monitor_list).values())
    logger.debug(json.dumps(response, indent=4))

    if monitor_id:
        for item in response:
            if item["id"] == monitor_id:
                console.print(f":cupcake: Details about monitor '{item['name']}'")
                console.print(json.dumps(item, indent=4, sort_keys=True), style="logging.level.info")
                return True
        console.print(f":four_leaf_clover: Monitor with ID {monitor_id} does not exist.", style="logging.level.error")
    else:
        table = Table("id", "name")
        if show_groups:
            console.print(f":hamburger: Available monitor groups.", style="green")
            for item in response:
                if item["type"] == "group":
                    table.add_row(str(item["id"]), item["name"])
            if table.rows:
                console.print(table, style="green")
            else:
                console.print(f":four_leaf_clover: No data available.")
        elif show_processes:
            for item in response:
                if item["type"] != "group":
                    table.add_row(str(item["id"]), item["name"])
            console.print(f":hamburger: Available monitor processes.", style="green")
            if table.rows:
                console.print(table, style="green")
            else:
                console.print(f":four_leaf_clover: No data available.", style="logging.level.info")
        elif verbose:
            console.print(json.dumps(response, indent=4, sort_keys=True))
        else:
            console.print(f":hamburger: Available monitors (groups and processes).", style="green")
            monitors = Columns(sorted([item['name'] for item in response], key=str.lower), equal=True, expand=True)
            console.print(monitors, style="logging.level.info")



#!/usr/bin/env python3

"""Status pages module for kumaone"""

# Import builtin python libraries
import json
import sys
import yaml

# Import external python libraries
from rich.console import Console
from rich.table import Table
import requests


# Import custom (local) python packages
from .event_handlers import get_event_data, wait_for_event
from . import ioevents
from .monitors import _check_monitor
from .payload_handler import _get_status_page_data_payload
from .settings import get_missing_arguments, timeout
from .utils import _sio_call

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()


def _get_status_page_public_group_list(public_group_list=None):
    """
    Generate public group list from monitor names

    :param public_group_list: (dict) Public group list dictionary
    :return: (list) Public group list of dictionary
    """

    # public_group_monitor_list = [
    #     {
    #         "name": "Services",
    #         "weight": 1,
    #         "monitorList": [
    #             {"id": 11}
    #         ]
    #     }
    # ]
    public_group_monitor_list = []
    if public_group_list is not None:
        for index, public_group in enumerate(public_group_list):
            public_group_monitor_item = {
                "name": public_group["name"],
                "weight": index + 1,
            }
            monitor_id_list = []
            for name in public_group["monitorList"]:
                monitor = _check_monitor(monitor_name_to_check=name)
                if monitor["exists"]:
                    monitor_id_list.append({"id": monitor["id"]})
            if monitor_id_list:
                public_group_monitor_item["monitorList"] = monitor_id_list
            public_group_monitor_list.append(public_group_monitor_item)
    return public_group_monitor_list


def list_status_pages(verbose=None, logger=None):
    """
    Show list of status pages.

    :param verbose: (bool) Enable verbose output.
    :param logger: (object) Logger object.
    :return: None
    """

    response = list(get_event_data(ioevents.status_page_list).values())
    logger.info(json.dumps(response, indent=4))
    console.print(f":hamburger: Available status pages.", style="green")
    table = Table("id", "slug", "Title")
    if not verbose:
        if response:
            for item in response:
                table.add_row(str(item["id"]), item["slug"], item["title"])
            if table.rows:
                console.print(table, style="green")
            else:
                console.print(f":four_leaf_clover: No data available.")
    else:
        console.print(json.dumps(response, indent=4, sort_keys=True), style="green")


def get_satus_page(slug=None, url=None, logger=None, show_details=False):
    """
    Get a status page by slug.

    :param slug: (str) Status page slug.
    :param url: (str) URL for uptime kuma server
    :param logger: (object) Logger object.
    :param show_details: (bool) Show details of the status page.
    :return: (dict) Python dictionary with status page info
    """

    event_response = _sio_call("getStatusPage", slug)

    try:
        http_response = requests.get(f"{url}/api/status-page/{slug}", timeout=timeout).json()
    except requests.exceptions.JSONDecodeError:
        console.print(f":cyclone: Response is not valid JSON.", style="logging.level.error")
        sys.exit(1)
    except requests.exceptions.Timeout:
        console.print(f":poodle: Response timed out.", style="logging.level.error")
        sys.exit(1)

    config = event_response["config"]
    config.update(http_response["config"])
    status_page_data = {
        **config,
        "incident": http_response["incident"],
        "publicGroupList": http_response["publicGroupList"],
        "maintenanceList": http_response["maintenanceList"],
    }
    # TODO: Check if we need to convert 'sendUrl' to boolean
    if show_details:
        console.print(f":page_facing_up: '{slug}' status page details", style="logging.level.info")
        console.print(f"{json.dumps(status_page_data, indent=4, sort_keys=True)}", style="logging.level.info")
    return status_page_data


def add_status_page(
    status_page_data_files=None, status_page_title=None, status_page_slug=None, logger=None, url=None, save=None
):
    """
    Creates/Adds a status page in uptime kuma.

    :param status_page_data_files: (Path) Status page config file location. File or Directory.
    :param status_page_title: (str) Title of the status page.
    :param status_page_slug: (str) Slug of the status page.
    :param logger: (object) Logger object for logging purposes.
    :param url: (str) URL of uptime kuma server
    :return: None.
    """

    if status_page_data_files:
        # console.print(Rule(style="purple"))
        for status_page_data_file in status_page_data_files:
            with open(status_page_data_file, "r") as tmp_data_file_read:
                status_pages = yaml.safe_load(tmp_data_file_read)["status_pages"]
                for status_page in status_pages:
                    logger.debug(status_page)
                    status_page_info = add_status_page(
                        status_page_title=status_page["title"].title(),
                        status_page_slug=status_page["slug"],
                        logger=logger,
                        url=url,
                    )
                    logger.debug(status_page_info)
                    if save:
                        if "ok" in status_page_info:
                            status_page_info.pop("ok")
                        if "msg" in status_page_info:
                            status_page_info.pop("msg")
                        status_page_info.update(status_page)
                        public_group_list = _get_status_page_public_group_list(status_page["publicGroupList"])
                        status_page_info["publicGroupList"] = public_group_list
                        status_page_data_to_save = _get_status_page_data_payload(**status_page_info)
                        logger.debug(status_page_data_to_save)
                        status_page_save_response = _sio_call("saveStatusPage", status_page_data_to_save)
                        if status_page_save_response["ok"]:
                            console.print(f":floppy_disk: Status page saved successfully!", style="logging.level.info")
                        else:
                            console.print(
                                f":cyclone: Status page ({status_page['slug']}) couldn't be saved. Error: {status_page_save_response.get('msg')}",
                                style="logging.level.error",
                            )
    else:
        status_page_info = _sio_call("getStatusPage", status_page_slug)
        if status_page_info["ok"]:
            status_page_details = get_satus_page(slug=status_page_slug, url=url, logger=logger)
            status_page_id = status_page_details["id"]
            console.print(
                f":sunflower: Status page '{status_page_id} - {status_page_title} ({status_page_slug})' already exists.",
                style="logging.level.info",
            )
            logger.debug(status_page_details)
            status_page_details.pop("incident")
            status_page_details.pop("maintenanceList")
            return status_page_details
        else:
            with wait_for_event(ioevents.status_page_list):
                response = _sio_call("addStatusPage", (status_page_title.title(), status_page_slug))
                if response["ok"]:
                    console.print(
                        f":hatching_chick: Status page '{status_page_title.title()} ({status_page_slug})' has been created.",
                        style="logging.level.info",
                    )
                    return response
                    logger.debug(f"Status page creation response: {response}")
                else:
                    console.print(f":red_circle: Error: {response['msg']}")
                    sys.exit(1)
    # console.print(Rule(style="purple"))


def delete_status_page(status_page_data_files=None, status_page_slug=None, logger=None):
    """
    Deletes a status page from uptime kuma.

    :param status_page_data_files: (Path) Status page config file location. File or Directory.
    :param status_page_slug: (str) Slug of the status page.
    :param logger: (object) Logger object for logging purposes.
    :return: None.
    """

    # console.print(Rule(title="Delete Status Page", style="purple"))
    if status_page_data_files:
        for status_page_data_file in status_page_data_files:
            with open(status_page_data_file, "r") as tmp_data_file_read:
                status_pages = yaml.safe_load(tmp_data_file_read)["status_pages"]
                for status_page in status_pages:
                    delete_status_page(status_page_slug=status_page["slug"], logger=logger)
    elif status_page_slug:
        status_page_info = _sio_call("getStatusPage", status_page_slug)
        if status_page_info["ok"]:
            response = _sio_call("deleteStatusPage", status_page_slug)
            if response["ok"]:
                console.print(
                    f":wastebasket: Status page '{status_page_slug}' has been deleted.", style="logging.level.info"
                )
        else:
            console.print(
                f":orange_circle: Status page '{status_page_slug}' does not exist.", style="logging.level.info"
            )



#!/usr/bin/env python3


"""Notification module for kumaone"""

# Import builtin python libraries
from pathlib import Path
from typing import Optional

# Import external python libraries
from rich.console import Console
import typer
from typing_extensions import Annotated

# Import custom (local) python packages
from src.kumaone.configs import check_config
from src.kumaone.connection import connect_login, disconnect
from src.kumaone.notifications import add_notification, delete_notification, list_notifications
from src.kumaone.utils import log_manager, _mutual_exclusivity_check

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

# Create typer app and turn off debug mode by default
app = typer.Typer()
state = {"log_level": "NOTSET"}
console = Console()


@app.command(name="add", help="Add a new notification provider.")
def notification_add(
    notifications: Annotated[
        Optional[Path],
        typer.Option(..., "--notifications", "-n", help="Notification(s) data."),
    ] = None,
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    interactive: Annotated[bool, typer.Option(help="Add notification interactively.")] = False,
    verbose: Annotated[bool, typer.Option(help="Show verbose output.")] = False,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Add a new notification provider.

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)
    if interactive and notifications:
        raise typer.BadParameter("Only one parameter is allowed.")

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    if interactive:
        add_notification(interactive=interactive, verbose=verbose, logger=logger)
    elif notifications:
        add_notification(notifications_file_path=notifications, interactive=interactive, verbose=verbose, logger=logger)
    disconnect()


@app.command(name="list", help="List all uptime kuma notification providers.")
def notification_list(
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    verbose: Annotated[bool, typer.Option(help="Show verbose output.")] = False,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    List all uptime kuma notification providers.

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    list_notifications(verbose=verbose, logger=logger)
    disconnect()


@app.command(name="show", help="Show details of an uptime kuma notification provider.")
def notification_show(
    notification_title: Annotated[
        str,
        typer.Option(
            ..., "--title", "-t", help="Uptime kuma notification name.", callback=_mutual_exclusivity_check(size=2)
        ),
    ] = None,
    notification_id: Annotated[
        int,
        typer.Option(
            ..., "--id", "-i", help="Uptime kuma notification id.", callback=_mutual_exclusivity_check(size=2)
        ),
    ] = None,
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    verbose: Annotated[bool, typer.Option(help="Show verbose output.")] = False,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    List all uptime kuma notification provider.

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)
    if notification_title is None and notification_id is None:
        raise typer.BadParameter("At least on of '--name' / '-n' or '--id' / '-i' parameter is required.")
    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    list_notifications(
        verbose=verbose, notification_title=notification_title, notification_id=notification_id, logger=logger
    )
    disconnect()


@app.command(name="delete", help="Delete a notification provider.")
def notification_delete(
    notifications: Annotated[
        Optional[Path],
        typer.Option(..., "--notifications", "-n", help="Notification(s) data."),
    ] = None,
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    notification_title: Annotated[
        str, typer.Option(..., "--title", "-t", help="Notification title (case sensitive).")
    ] = None,
    verbose: Annotated[bool, typer.Option(help="Show verbose output.")] = False,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Delete a notification provider.

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)
    if notification_title and notifications:
        raise typer.BadParameter(message="Only one parameter is allowed.")
    if notification_title is None and notifications is None:
        raise typer.BadParameter(message="At least one of '--notifications' / '--name' parameter is required.")

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    if notification_title:
        delete_notification(notification_title=notification_title, verbose=verbose, logger=logger)
    elif notifications:
        delete_notification(
            notifications_file_path=notifications, notification_title=notification_title, verbose=verbose, logger=logger
        )
    disconnect()


@app.callback()
def notification_mission_control(log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET"):
    """
    Kumaone notification process manager. An uptime kuma notification processes manager wrapper.
    """

    if log_level:
        state["log_level"] = log_level


if __name__ == "__main__":
    app()



#!/usr/bin/env python3

"""Payload handler module for kumaone"""

# Import builtin python libraries
import logging
import sys

# Import external python libraries
from rich.console import Console

# Import custom (local) python packages
from .notification_settings import notification_providers
from .settings import authentication_methods

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()


def _get_monitor_payload(
    type=None,
    name=None,
    parent=None,
    description=None,
    interval=60,
    retryInterval=60,
    resendInterval=0,
    maxretries=1,
    upsideDown=False,
    notificationIDList=None,
    httpBodyEncoding="json",
    # HTTP, KEYWORD, JSON_QUERY, REAL_BROWSER
    url=None,
    # HTTP, KEYWORD, GRPC_KEYWORD
    maxredirects=10,
    accepted_statuscodes=None,
    # HTTP, KEYWORD, JSON_QUERY
    expiryNotification=False,
    ignoreTls=False,
    proxyId=None,
    method="GET",
    body=None,
    headers=None,
    authMethod="",
    tlsCert=None,
    tlsKey=None,
    tlsCa=None,
    basic_auth_user=None,
    basic_auth_pass=None,
    authDomain=None,
    authWorkstation=None,
    oauth_auth_method="client_secret_basic",
    oauth_token_url=None,
    oauth_client_id=None,
    oauth_client_secret=None,
    oauth_scopes=None,
    timeout=48,
    # KEYWORD
    keyword=None,
    invertKeyword=False,
    # GRPC_KEYWORD
    grpcUrl=None,
    grpcEnableTls=False,
    grpcServiceName=None,
    grpcMethod=None,
    grpcProtobuf=None,
    grpcBody=None,
    grpcMetadata=None,
    # PORT, PING, DNS, STEAM, MQTT, RADIUS, TAILSCALE_PING
    hostname=None,
    # PING
    packetSize=56,
    # PORT, DNS, STEAM, MQTT, RADIUS
    port=None,
    # DNS
    dns_resolve_server="1.1.1.1",
    dns_resolve_type="A",
    # MQTT
    mqttUsername="",
    mqttPassword="",
    mqttTopic="",
    mqttSuccessMessage="",
    # SQLSERVER, POSTGRES, MYSQL, MONGODB, REDIS
    databaseConnectionString=None,
    # SQLSERVER, POSTGRES, MYSQL
    databaseQuery=None,
    # DOCKER
    docker_container="",
    docker_host=None,
    # RADIUS
    radiusUsername=None,
    radiusPassword=None,
    radiusSecret=None,
    radiusCalledStationId=None,
    radiusCallingStationId=None,
    # GAMEDIG
    game=None,
    gamedigGivenPortOnly=True,
    # JSON_QUERY
    jsonPath=None,
    expectedValue=None,
    # KAFKA_PRODUCER
    kafkaProducerBrokers=None,
    kafkaProducerTopic=None,
    kafkaProducerMessage=None,
    kafkaProducerSsl=False,
    kafkaProducerAllowAutoTopicCreation=False,
    kafkaProducerSaslOptions=None,
):
    """
    Generates payload for monitor "add" event

    :return: (dict) Python dictionary for payload
    """

    if accepted_statuscodes is None:
        accepted_statuscodes = ["200-299"]

    if notificationIDList is None:
        notificationIDList = []

    monitor_data = {
        "type": type,
        "name": name,
        "interval": interval,
        "retryInterval": retryInterval,
        "maxretries": maxretries,
        "notificationIDList": notificationIDList,
        "upsideDown": upsideDown,
        "resendInterval": resendInterval,
        "description": description,
        "httpBodyEncoding": httpBodyEncoding,
        "parent": parent,
        "expiryNotification": expiryNotification,
        "ignoreTls": ignoreTls,
        "proxyId": proxyId,
        "method": method,
        "body": body,
        "headers": headers,
        "authMethod": authMethod,
        "timeout": timeout,
    }

    # HTTP, KEYWORD, JSON_QUERY, REAL_BROWSER
    monitor_data.update({"url": url})

    if type in ["keyword", "grpc-keyword"]:
        monitor_data.update(
            {
                "keyword": keyword,
                "invertKeyword": invertKeyword,
            }
        )

    # HTTP, KEYWORD, GRPC_KEYWORD
    monitor_data.update(
        {
            "maxredirects": maxredirects,
            "accepted_statuscodes": accepted_statuscodes,
        }
    )

    # AUTH METHODS
    if authMethod:
        if authMethod in authentication_methods:
            if authMethod in ["basic", "ntlm"]:
                monitor_data.update(
                    {
                        "basic_auth_user": basic_auth_user,
                        "basic_auth_pass": basic_auth_pass,
                    }
                )

            if authMethod == "ntlm":
                monitor_data.update(
                    {
                        "authDomain": authDomain,
                        "authWorkstation": authWorkstation,
                    }
                )

            if authMethod == "mtls":
                monitor_data.update(
                    {
                        "tlsCert": tlsCert,
                        "tlsKey": tlsKey,
                        "tlsCa": tlsCa,
                    }
                )

            if authMethod == "oauth2-cc":
                monitor_data.update(
                    {
                        "oauth_auth_method": oauth_auth_method,
                        "oauth_token_url": oauth_token_url,
                        "oauth_client_id": oauth_client_id,
                        "oauth_client_secret": oauth_client_secret,
                        "oauth_scopes": oauth_scopes,
                    }
                )
        else:
            console.print(
                f":blowfish: Provided authentication method: {authMethod} is not supported.", style="logging.level.info"
            )

    if type == "grpc-keyword":
        monitor_data.update(
            {
                "grpcUrl": grpcUrl,
                "grpcEnableTls": grpcEnableTls,
                "grpcServiceName": grpcServiceName,
                "grpcMethod": grpcMethod,
                "grpcProtobuf": grpcProtobuf,
                "grpcBody": grpcBody,
                "grpcMetadata": grpcMetadata,
            }
        )

    # DNS, MQTT, PING, PORT, RADIUS, STEAM, TAILSCALE_PING
    if type in ["dns", "mqtt", "ping", "port", "radius", "steam", "tailscale-ping"]:
        monitor_data.update(
            {
                "hostname": hostname,
            }
        )

    # PING
    if type == "ping":
        monitor_data.update(
            {
                "packetSize": packetSize,
            }
        )

    # PORT, DNS, STEAM, MQTT, RADIUS
    if not port:
        if type == "radius":
            port = 1812
    monitor_data.update(
        {
            "port": port,
        }
    )

    # DNS
    if type == "dns":
        monitor_data.update(
            {
                "dns_resolve_server": dns_resolve_server,
                "dns_resolve_type": dns_resolve_type,
            }
        )
        if not port:
            monitor_data.update({"port": 53})

    # MQTT
    if type == "mqtt":
        monitor_data.update(
            {
                "mqttUsername": mqttUsername,
                "mqttPassword": mqttPassword,
                "mqttTopic": mqttTopic,
                "mqttSuccessMessage": mqttSuccessMessage,
            }
        )

    # SQLSERVER, POSTGRES, MYSQL, MONGODB, REDIS
    if type in ["sqlserver", "postgres", "mysql", "mongodb", "redis"]:
        monitor_data.update({"databaseConnectionString": databaseConnectionString})

    # SQLSERVER, POSTGRES, MYSQL
    if type in ["sqlserver", "postgres", "mysql"]:
        monitor_data.update(
            {
                "databaseQuery": databaseQuery,
            }
        )

    # DOCKER
    if type == "docker":
        monitor_data.update(
            {
                "docker_container": docker_container,
                "docker_host": docker_host,
            }
        )

    # RADIUS
    if type == "radius":
        monitor_data.update(
            {
                "radiusUsername": radiusUsername,
                "radiusPassword": radiusPassword,
                "radiusSecret": radiusSecret,
                "radiusCalledStationId": radiusCalledStationId,
                "radiusCallingStationId": radiusCallingStationId,
            }
        )

    # GAMEDIG
    if type == "gamedig":
        monitor_data.update(
            {
                "game": game,
                "gamedigGivenPortOnly": gamedigGivenPortOnly,
            }
        )

    # JSON_QUERY
    if type == "json-query":
        monitor_data.update(
            {
                "jsonPath": jsonPath,
                "expectedValue": expectedValue,
            }
        )

    # KAFKA_PRODUCER
    if type == "kafka-producer":
        if kafkaProducerBrokers is None:
            kafkaProducerBrokers = []
        if not kafkaProducerSaslOptions:
            kafkaProducerSaslOptions = {
                "mechanism": "None",
            }
        monitor_data.update(
            {
                "kafkaProducerBrokers": kafkaProducerBrokers,
                "kafkaProducerTopic": kafkaProducerTopic,
                "kafkaProducerMessage": kafkaProducerMessage,
                "kafkaProducerSsl": kafkaProducerSsl,
                "kafkaProducerAllowAutoTopicCreation": kafkaProducerAllowAutoTopicCreation,
                "kafkaProducerSaslOptions": kafkaProducerSaslOptions,
            }
        )

    return monitor_data


def _get_status_page_data_payload(
    slug=None,
    id=None,
    title=None,
    description=None,
    theme="auto",
    published=True,
    showTags=False,
    domainNameList=None,
    googleAnalyticsId=None,
    customCSS="",
    footerText=None,
    showPoweredBy=False,
    showCertificateExpiry=False,
    icon="/icon.svg",
    publicGroupList=None,
):
    """
    Generates payload data for status page "save" event

    :param slug: (str) Status page slug.
    :param id: (int) Status page ID.
    :param title: (str) Status page title.
    :param description: (str) Status page description.
    :param theme: (str) Status page theme. [auto], light, or dark theme.
    :param published: (bool) Status page status. Default [True].
    :param showTags: (bool) Should the tags be visible on the page. Default [False].
    :param domainNameList: (list) List of domain names.
    :param googleAnalyticsId: (str) Google Analytics ID.
    :param customCSS: (str) Custom CSS.
    :param footerText: (str) Footer text for status page.
    :param showPoweredBy: (bool) Show powered by. Default [False].
    :param showCertificateExpiry: (bool) Show the status page show SSL certificate expiry time.
    :param icon: (str) Status page icon location path. Default ["/icon.svg"]
    :param publicGroupList: (list) List of public groups.
    :return: (dict) Status page save payload data
    """

    if not domainNameList:
        domainNameList = []
    if not publicGroupList:
        publicGroupList = []

    status_page_payload = {
        "id": id,
        "slug": slug,
        "title": title,
        "description": description,
        "domainNameList": domainNameList,
        "icon": icon,
        "theme": theme,
        "published": published,
        "showTags": showTags,
        "showPoweredBy": showPoweredBy,
        "googleAnalyticsId": googleAnalyticsId,
        "customCSS": customCSS,
        "footerText": footerText,
        "showCertificateExpiry": showCertificateExpiry,
    }

    return slug, status_page_payload, icon, publicGroupList



#!/usr/bin/env python3

"""Notification pages module for kumaone"""

# Import builtin python libraries
import json
from pathlib import Path
import yaml

# Import external python libraries
from rich.console import Console
from rich.table import Table

# Import custom (local) python packages
from .event_handlers import get_event_data, wait_for_event
from . import ioevents
from .utils import _sio_call

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

console = Console()


def _get_notification_by_name_or_id(response=None, notification_title=None, notification_id=None, logger=None):
    """
    Get notification process details by name or id

    :param response: (list) List of dictionaries with notification data.
    :param notification_title: (str) The name of the notification process.
    :param notification_id: (int) id of the notification process.
    :param logger: (object) Logger object instance.
    :return: (dict) notification details.
    """

    for _notification in response:
        logger.debug(f"Notification id: {notification_id}")
        logger.debug(f"Notification name: {notification_title}")
        if _notification["name"] == notification_title or _notification["id"] == notification_id:
            response = [_notification]
            return response


def _get_notification_id_by_name(notification_title=None, logger=None):
    """
    Get notification id by name.

    :param notification_title: (str) Name of the notification.
    :param logger: (object) Logger object instance.
    :return: (int) Notification id.
    """

    response = get_event_data(ioevents.notification_list)
    logger.debug(f"Notification List: {response}")
    for _notification in response:
        if _notification["name"] == notification_title:
            return _notification["id"]
    return -1


def add_notification(notifications_file_path=None, interactive=None, logger=None, verbose=None):
    """
    Add notification provider(s)

    :param notifications_file_path: (Path) Path to the notification provider definition file.
    :param interactive: (bool) Whether the notification provider should be added interactively or not.
    :param logger: (object) Logger object instance.
    :param verbose: (bool) Print verbose output if True.
    :return: None
    """

    if notifications_file_path is not None and Path(notifications_file_path).is_file():
        with open(notifications_file_path) as notification_config_file:
            notification_configs = yaml.safe_load(notification_config_file)["notifications"]
        logger.debug(notification_configs)
    for notification_config in notification_configs:
        for payload in notification_config.values():
            logger.debug(f"Payload for '{payload['type']}': {payload}")
            notification_provider_exists = list_notifications(
                notification_title=payload["name"], logger=logger, check_existence=True
            )
            if not notification_provider_exists:
                with wait_for_event(ioevents.notification_list):
                    response = _sio_call("addNotification", (payload, None))
                    if verbose:
                        print(f"{payload['type']}: {response}")
                    else:
                        if response["ok"]:
                            console.print(
                                f":floppy_disk: Notification provider '{payload['type']}' added successfully.",
                                style="logging.level.info",
                            )
                        else:
                            console.print(
                                f":orange_circle: '{payload['type']}' notification provider addition failed. Response: {response['msg']}",
                                style="logging.level.warning",
                            )
            else:
                console.print(f":sunflower: '{payload['type']}' notification provider already exists.")


def list_notifications(verbose=None, notification_title=None, notification_id=None, logger=None, check_existence=False):
    """
    Show list of notification processes

    :param verbose: (bool) Enable verbose output.
    :param notification_title: (str) Uptime kuma notification process name.
    :param notification_id: (int) Uptime kuma notification process id.
    :param logger: (object) Logging object.
    :param check_existence: (bool) Check existence of notification provider by name.
    :return: None
    """

    response = get_event_data(ioevents.notification_list)
    logger.debug(json.dumps(response, indent=4, sort_keys=True))
    # pretty print information
    pretty_response = []
    for notification in response:
        flat_notification = notification.copy()
        config = json.loads(flat_notification["config"])
        del flat_notification["config"]
        flat_notification.update(config)
        pretty_response.append(flat_notification)
    if notification_title is not None or notification_id is not None:
        pretty_response = _get_notification_by_name_or_id(
            response=pretty_response,
            notification_title=notification_title,
            notification_id=notification_id,
            logger=logger,
        )
    if pretty_response:
        if check_existence:
            return True
        console.print(f":megaphone: Available notification providers", style="logging.level.info")
        if verbose:
            console.print(f"{json.dumps(pretty_response, indent=4, sort_keys=True)}", style="logging.level.info")
        else:
            table = Table("id", "type", "title", "active", "isDefault")
            for item in pretty_response:
                table.add_row(str(item["id"]), item["type"], item["name"], str(item["active"]), str(item["isDefault"]))
            if table.rows:
                console.print(table, style="green")
            else:
                console.print(f":four_leaf_clover: No data available.", style="logging.level.info")
    else:
        if check_existence:
            return False
        console.print(f":four_leaf_clover: No data available.", style="logging.level.info")


def delete_notification(notifications_file_path=None, notification_title=None, logger=None, verbose=None):
    """
    Delete a notification provider.

    :param notifications_file_path: (Path) Path to the notification provider definition file.
    :param notification_title: (str) Whether the notification provider should be added interactively or not.
    :param logger: (object) Logger object instance.
    :param verbose: (bool) Print verbose output if True.
    :return: None
    """

    notification_provider_to_delete = []
    if notifications_file_path is not None and Path(notifications_file_path).is_file():
        with open(notifications_file_path, "r") as notification_config_file:
            notification_configs = yaml.safe_load(notification_config_file)["notifications"]
        logger.debug(notification_configs)
        for notification_config in notification_configs:
            for _notification in notification_config.values():
                notification_id = _get_notification_id_by_name(notification_title=_notification["name"], logger=logger)
                if notification_id != -1:
                    notification_provider_to_delete.append(notification_id)
    elif notification_title is not None:
        notification_id = _get_notification_id_by_name(notification_title=notification_title, logger=logger)
        if notification_id != -1:
            notification_provider_to_delete.append(notification_id)
    logger.debug(f"Notification provider IDs to delete: {notification_provider_to_delete}")
    if notification_provider_to_delete:
        for _notification_id in notification_provider_to_delete:
            with wait_for_event(ioevents.notification_list):
                delete_event_response = _sio_call("deleteNotification", _notification_id)
                if delete_event_response["ok"]:
                    console.print(
                        f":ghost: Notification provider with id: '{_notification_id}' has been deleted.",
                        style="logging.level.info",
                    )
    else:
        console.print(f":running_shoe: Notification provider(s) doesn't exist. Skipping...", style="logging.level.info")



#!/usr/bin/env python3


"""monitor module for kumaone"""

# Import builtin python libraries
from pathlib import Path
from typing import Optional

# Import external python libraries
from rich.console import Console
from typing_extensions import Annotated
import typer

# Import custom (local) python packages
from src.kumaone.configs import check_config
from src.kumaone.connection import connect_login, disconnect
from src.kumaone.monitors import add_monitor, delete_monitor, list_monitors
from src.kumaone.utils import _check_data_path, log_manager, _mutual_exclusivity_check

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

# Create typer app and turn off debug mode by default
app = typer.Typer()
state = {"log_level": "NOTSET"}
console = Console()


@app.command(name="add", help="Add one or more monitor(s).")
def monitor_add(
    monitors: Annotated[Optional[Path], typer.Option(..., "--monitors", "-m", help="Monitor(s) data.")],
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Adds uptime kuma monitor(s)

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    monitor_file_paths = _check_data_path(data_path=monitors, logger=logger, key_to_check_for="monitors")
    add_monitor(monitor_data_files=monitor_file_paths, logger=logger)
    disconnect()


@app.command(name="delete", help="Delete one or more monitor(s).")
def monitor_delete(
    monitors: Annotated[
        Optional[Path],
        typer.Option(..., "--monitors", "-m", help="Monitor(s) data.", callback=_mutual_exclusivity_check(size=2)),
    ] = None,
    monitor_name: Annotated[
        str,
        typer.Option(
            ..., "--name", "-n", help="Name of the monitor to delete.", callback=_mutual_exclusivity_check(size=2)
        ),
    ] = None,
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Deletes uptime kuma monitor(s).

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    if monitors is None and monitor_name is None:
        raise typer.BadParameter("At least one of the parameter '--monitor'/'-m' OR '--name'/'-n' is required.")
    if monitors:
        monitor_file_paths = _check_data_path(data_path=monitors, logger=logger, key_to_check_for="monitors")
        delete_monitor(monitor_data_files=monitor_file_paths, logger=logger)
    elif monitor_name:
        delete_monitor(monitor_name=monitor_name, logger=logger)
    disconnect()


@app.command(name="list", help="List all monitor groups and processes.")
def monitor_list(
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    groups: Annotated[bool, typer.Option(help="Show only monitoring groups.")] = False,
    processes: Annotated[bool, typer.Option(help="Show only monitoring processes.")] = False,
    verbose: Annotated[bool, typer.Option(help="Show verbose output.")] = False,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Lists all monitor groups and processes

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    if groups and processes:
        raise typer.BadParameter(message="'--groups' and '--processes' can not be used together.")
    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    list_monitors(show_groups=groups, show_processes=processes, verbose=verbose, logger=logger)
    disconnect()


@app.command(name="show", help="Show details of a single process monitor by ID.")
def monitor_show(
    monitor_id: Annotated[int, typer.Option(..., "--id", "-i", help="Uptime kuma monitor ID.")],
    config_file: Annotated[
        Optional[Path], typer.Option(..., "--config", "-c", help="Uptime kuma configuration file path.")
    ] = Path.home().joinpath(".config/kumaone/kuma.yaml"),
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Lists all monitor groups and processes

    :return: None
    """

    if log_level:
        state["log_level"] = log_level
        logger = log_manager(log_level=log_level)

    config_data = check_config(config_path=config_file, logger=logger)
    connect_login(config_data=config_data)
    list_monitors(monitor_id=monitor_id, logger=logger)
    disconnect()


@app.callback()
def monitor_mission_control(log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET"):
    """
    Kumaone monitor manager. An uptime kuma monitor group and process monitor manager wrapper.
    """

    if log_level:
        state["log_level"] = log_level


if __name__ == "__main__":
    app()



#!/usr/bin/env python3

"""Main module for kumaone"""

# Import builtin python libraries

# Import external python libraries
from rich.console import Console
import typer
from typing_extensions import Annotated
from typing import Optional

# Import custom (local) python packages
from src.kumaone.cli import config_cli, monitor_cli, notification_cli, status_page_cli
from src.kumaone.utils import app_info, version_callback

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

# Create typer app and turn off debug mode by default
app = typer.Typer()
app.add_typer(monitor_cli.app, name="monitor")
app.add_typer(config_cli.app, name="config")
app.add_typer(status_page_cli.app, name="status-page")
app.add_typer(notification_cli.app, name="notification")
state = {"log_level": "NOTSET"}
console = Console()


@app.command(name="info", help="Show information about kumaone application.")
def info(log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET"):
    """
    Show application information

    :return: (None) Information on screen.
    """

    if log_level:
        state["log_level"] = log_level
    app_info(log_level=log_level)


@app.callback()
def mission_control(
    version: Annotated[Optional[bool], typer.Option("--version", callback=version_callback, is_eager=True)] = None,
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Mission control for kumaone, an uptime kuma helper python package.
    """

    if log_level:
        state["log_level"] = log_level


if __name__ == "__main__":
    app()



#!/usr/bin/env python3


"""monitor module for kumaone"""

# Import external python libraries
from rich.console import Console
import typer
from typing_extensions import Annotated

# Import custom (local) python packages
from src.kumaone.configs import check_config, create_config, delete_config

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"

# Create typer app and turn off debug mode by default
app = typer.Typer()
state = {"log_level": "NOTSET"}
console = Console()


@app.command(name="show", help="Show kumaone config.")
def config_show(
    config_path: Annotated[
        str, typer.Option(..., "--config", "-c", help="Custom full (with extension) location for config file.")
    ] = "",
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Show kumaone config for uptime kuma server.

    :return: (None) On screen output.
    """

    if log_level != "NOTSET":
        state["log_level"] = log_level

    check_config(config_path=config_path, log_level=log_level)


@app.command(name="create", help="Create kumaone config.")
def config_create(
    config_path: Annotated[
        str, typer.Option(..., "--config", "-c", help="Custom full (with extension) location for config file.")
    ] = "",
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Create kumaone config for uptime kuma server.

    :return: (None) On screen output.
    """

    if log_level != "NOTSET":
        state["log_level"] = log_level

    create_config(config_path=config_path, log_level=log_level)


@app.command(name="delete", help="Delete kumaone config.")
def config_delete(
    config_path: Annotated[
        str, typer.Option(..., "--config", "-c", help="Custom full (with extension) location for config file.")
    ] = "",
    log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
):
    """
    Delete kumaone config for uptime kuma server.

    :return: (None) On screen output.
    """

    if log_level != "NOTSET":
        state["log_level"] = log_level

    delete_config(config_path=config_path, log_level=log_level)


# @app.command(name="edit", help="Edit kumaone config.")
# def config_edit(
#     config_path: Annotated[
#         str, typer.Option(..., "--config", "-c", help="Custom full (with extension) location for config file.")
#     ] = "",
#     log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET",
# ):
#     """
#     Edit kumaone config for uptime kuma server.
#
#     :return: (None) On screen output.
#     """
#
#     if log_level != "NOTSET":
#         state["log_level"] = log_level
#
#     edit_config(config_path=config_path, log_level=log_level)


@app.callback()
def config_mission_control(log_level: Annotated[str, typer.Option(help="Set log level.")] = "NOTSET"):
    """
    Kumaone configuration manager.
    """

    if log_level:
        state["log_level"] = log_level


if __name__ == "__main__":
    app()



#!/usr/bin/env python3

"""SocketIO events module for kumaone"""

# Source code meta data
__author__ = "Dalwar Hossain"
__email__ = "dalwar23@pm.me"


api_key_list = "apiKeyList"
auto_login = "autoLogin"
avg_ping = "avgPing"
cert_info = "certInfo"
connect = "connect"
disconnect = "disconnect"
docker_host_list = "dockerHostList"
heartbeat = "heartbeat"
heartbeat_list = "heartbeatList"
important_heartbeat_list = "importantHeartbeatList"
info = "info"
init_server_timezone = "initServerTimezone"
maintenance_list = "maintenanceList"
monitor_list = "monitorList"
notification_list = "notificationList"
proxy_list = "proxyList"
status_page_list = "statusPageList"
uptime = "uptime"
